﻿using Blazor.Models;
using Blazored.LocalStorage;
using Microsoft.AspNetCore.Components;

namespace Blazor.Services
{
    public class DataLocalService : IDataService
    {
        private readonly HttpClient _http;
        private readonly ILocalStorageService _localStorage;
        private readonly NavigationManager _navigationManager;
        private readonly IWebHostEnvironment _webHostEnvironment;

        public DataLocalService(
            ILocalStorageService localStorage,
            HttpClient http,
            IWebHostEnvironment webHostEnvironment,
            NavigationManager navigationManager)
        {
            _localStorage = localStorage;
            _http = http;
            _webHostEnvironment = webHostEnvironment;
            _navigationManager = navigationManager;
        }

        public async Task<Chapter> GetById(int id)
        {
            // Get the current data
            var currentData = await _localStorage.GetItemAsync<List<Chapter>>("data");

            // Get the chapter int the list
            var chapter = currentData.FirstOrDefault(w => w.Id == id);

            // Check if chapter exist
            if (chapter == null)
            {
                throw new Exception($"Unable to found the item with ID: {id}");
            }

            return chapter;
        }

        public async Task Update(int id, ChapterModel model)
        {
            // Get the current data
            var currentData = await _localStorage.GetItemAsync<List<Chapter>>("data");

            // Get the chapter int the list
            var chapter = currentData.FirstOrDefault(w => w.Id == id);

            // Check if chapter exist
            if (chapter == null)
            {
                throw new Exception($"Unable to found the item with ID: {id}");
            }

            // Modify the content of the chapter
            chapter.Name = model.Name;

            // Save the data
            await _localStorage.SetItemAsync("data", currentData);
        }


        public async Task Add(ChapterModel model)
        {
            // Get the current data
            var currentData = await _localStorage.GetItemAsync<List<Chapter>>("data");

            // Simulate the Id
            model.Id = currentData.Max(s => s.Id) + 1;

            // Add the chapter to the current data
            currentData.Add(new Chapter
            {
                Id = model.Id,
                Name = model.Name
            });

          
            // Save the data
            await _localStorage.SetItemAsync("data", currentData);
        }


        public async Task<int> Count()
        {
            // Load data from the local storage
            var currentData = await _localStorage.GetItemAsync<Chapter[]>("data");

            // Check if data exist in the local storage
            if (currentData == null)
            {
                // this code add in the local storage the fake data
                var originalData = await _http.GetFromJsonAsync<Chapter[]>($"{_navigationManager.BaseUri}fake-data.json");
                await _localStorage.SetItemAsync("data", originalData);
            }

            return (await _localStorage.GetItemAsync<Chapter[]>("data")).Length;
        }

        public async Task<List<Chapter>> List(int currentPage, int pageSize)
        {
            // Load data from the local storage
            var currentData = await _localStorage.GetItemAsync<Chapter[]>("data");

            // Check if data exist in the local storage
            if (currentData == null)
            {
                // this code add in the local storage the fake data
                var originalData = await _http.GetFromJsonAsync<Chapter[]>($"{_navigationManager.BaseUri}fake-data.json");
                await _localStorage.SetItemAsync("data", originalData);
            }

            return (await _localStorage.GetItemAsync<Chapter[]>("data")).Skip((currentPage - 1) * pageSize).Take(pageSize).ToList();
        }
    }
}
